import pygame 
import os
import time 
import random 
import sys
#from MENU import *

pygame.font.init()

ancho,alto = 600,600

pantalla = pygame.display.set_mode((ancho,alto))
pygame.display.set_caption("Space invader 2")

#ENEMIGOS
NAVEROJA = pygame.image.load('naveroja.png')
NAVEAZUL = pygame.image.load('naveazul.png')
NAVEVERDE = pygame.image.load('naveverde.png')

#JUGADOR
NAVEAMARILLA = pygame.image.load('naveamarilla.png')

#DISPAROS
LASERED = pygame.image.load('laserred.png')
LASERBLUE = pygame.image.load('laserblue.png')
LASERGREEN = pygame.image.load('lasergreen.png')
LASERYELLOW = pygame.image.load('laseryellow.png')

#ESTETICA
FONDO =pygame.transform.scale(pygame.image.load('nuevo.png'), (ancho,alto))
MENU =pygame.transform.scale(pygame.image.load('menu.png'), (ancho,alto))
GOVER =pygame.transform.scale(pygame.image.load('gameover.png'), (ancho,alto))
TUTORIAL =pygame.transform.scale(pygame.image.load('tutorial.png'), (ancho,alto))
pygame.mixer.init()

#COLORES
white = (255,255,255)
purple = (213,35,226)
verde = (51,255,51)
negro = (0,0,0,0)

class Laser:
    def __init__(self, x, y, img):
        self.x = x
        self.y = y 
        self.img = img 
        self.mask = pygame.mask.from_surface(self.img)
    

    def dibujar(self, pantalla):
        pantalla.blit(self.img, (self.x,self.y))

    def movimiento(self, velocidad):
        self.y += velocidad 

    def apagado(self, alto):
        return not (self.y <= alto and self.y >= 0)

    def colision(self, objecto):
        return chocar(self, objecto)

class Nave:
    FRIO = 30

    def __init__(self, x, y, vida=100):
        self.x = x
        self.y = y
        self.vida = vida
        self.nave_img = None
        self.laser_img = None
        self.lasers = []
        self.cool_down_counter = 0 
        self.explo_img = None

    def dibujar(self , pantalla):
        pantalla.blit(self.nave_img, (self.x, self.y))
        for laser in self.lasers:
            laser.dibujar(pantalla)

    def mover_lasers(self,velocidad,objecto):
        self.frio()
        for laser in self.lasers:
            laser.movimiento(velocidad)
            if laser.apagado(alto):
                self.lasers.remove(laser)
            elif laser.colision(objecto):
                objecto.vida -= 10
                self.lasers.remove(laser)

    def frio(self):
        if self.cool_down_counter >= self.FRIO:
            self.cool_down_counter = 0
        elif self.cool_down_counter > 0:
            self.cool_down_counter += 1

    def disparo(self):
        if self.cool_down_counter == 0:
            laser = Laser(self.x,self.y,self.laser_img)

            
            self.lasers.append(laser)
            
            self.cool_down_counter = 5
            pshot=pygame.mixer.Sound("pew.wav")
            pshot.play()
            pshot.set_volume(0.2)

    def get_ancho(self):
        return self.nave_img.get_width()

    def get_altura(self):
        return self.nave_img.get_height()

class Jugador(Nave):
    def __init__(self, x, y, vida=100):
        super().__init__(x, y, vida)
        self.nave_img = NAVEAMARILLA
        self.laser_img = LASERYELLOW
        self.mask = pygame.mask.from_surface(self.nave_img)
        self.max_vida = vida

    def mover_lasers(self,velocidad,objectos):
        self.frio()
        for laser in self.lasers:
            laser.movimiento(velocidad)
            if laser.apagado(alto):
                self.lasers.remove(laser)
            else:
                for objecto in objectos:
                    if laser.colision(objecto):
                        objectos.remove(objecto)
                        self.lasers.remove(laser)


    def dibujar(self, pantalla):
        super().dibujar(pantalla)
        self.barra(pantalla)

    def barra(self, pantalla):
        pygame.draw.rect(pantalla, (255,0,0), (self.x, self.y + self.nave_img.get_height() + 10, self.nave_img.get_width(), 10))
        pygame.draw.rect(pantalla, (0,255,0), (self.x, self.y + self.nave_img.get_height() + 10, self.nave_img.get_width() * (self.vida/self.max_vida), 10))

class Enemigo(Nave):
    COLOR_MAP ={
                "red" : (NAVEROJA, LASERED),
                "green": (NAVEVERDE, LASERGREEN),
                "blue": (NAVEAZUL, LASERBLUE)
                }

    def __init__(self, x, y, color, vida=100):
        super().__init__(x,y,vida)
        self.nave_img, self.laser_img = self.COLOR_MAP[color]
        self.mask = pygame.mask.from_surface(self.nave_img)

    def movimiento_enemigo(self, velocidad):
        self.y += velocidad

    def disparo(self):
        if self.cool_down_counter == 0:
            laser = Laser(self.x-20, self.y,self.laser_img)
            self.lasers.append(laser)
            self.cool_down_counter = 1

def chocar(objecto1 , objecto2):
    compensar_x = objecto2.x - objecto1.x
    compensar_y = objecto2.y - objecto1.y
    return objecto1.mask.overlap(objecto2.mask, (compensar_x, compensar_y)) != None

def SpaceInvader2():
    pygame.mixer.init()
    pygame.mixer.music.load("musica.ogg")
    pygame.mixer.music.play(-1) 
    getready=pygame.mixer.Sound("getready.ogg")
    getready.play()
    getready.set_volume(0.1)
    correr = True
    FPS = 60
    nivel = 0
    niveles = 5
    SpaceInvader2_font = pygame.font.SysFont("comicsans",50)
    perdio_font = pygame.font.SysFont("comicsans", 60)
    rango = 5
    enemigos = []
    enemigo_vel = 5

    jugador = Jugador(250,250)
    jugador_vel = 15
    clock = pygame.time.Clock()
    laser_vel = 15
    perdio = False
    perdio_contador = 0

    FPSfondo=70
    y = 0
    puntos = 0


    def redibujar_window():

        nivel_label = SpaceInvader2_font.render(f"nivel : {nivel}",1 ,(255,255,255))

        pantalla.blit(nivel_label, (ancho - nivel_label.get_width() - 10, 10))

        for Enemigo in enemigos:
            Enemigo.dibujar(pantalla)

        jugador.dibujar(pantalla)
        

        if perdio:
            gaover=pygame.mixer.Sound("gameover.ogg")
            gaover.play()
            gaover.set_volume(0.1)
            pantalla.blit(GOVER,(0,0))


        pygame.display.update()

    while correr:
        clock.tick(FPS)
        redibujar_window()
        




        if niveles <= 0 or jugador.vida <= 0:
            enemydied=pygame.mixer.Sound("playerexpl.wav")
            enemydied.play()
            enemydied.set_volume(0.2)
            perdio = True
            perdio_contador += 1

        if perdio:
            if perdio_contador > FPS * 3:
                correr = False
                
            else:
                continue

        if len(enemigos) == 0:
            nivel += 1
            rango += 5
            for i in range(rango):
                enemigo = Enemigo(random.randrange(50, ancho-100), random.randrange(-1500, -100), random.choice(["red","blue","green"]))
                enemigos.append(enemigo)


        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()



        y_relativa = y % FONDO.get_rect().height
        pantalla.blit(FONDO, (0, y_relativa - FONDO.get_rect().height))
        if y_relativa < alto:
            pantalla.blit(FONDO, (0,y_relativa))
        y +=1
        clock.tick(FPSfondo)

        keys = pygame.key.get_pressed()
        if keys[pygame.K_LEFT] and jugador.x + jugador_vel > 0: # izquierda
            jugador.x -= jugador_vel
        if keys[pygame.K_RIGHT] and jugador.x + jugador_vel + jugador.get_ancho() < ancho: #derecha
            jugador.x += jugador_vel
        if keys[pygame.K_UP] and jugador.y + jugador_vel > 0: # arriba 
            jugador.y -= jugador_vel
        if keys[pygame.K_DOWN] and jugador.y + jugador_vel + jugador.get_altura() < alto: #abajo
            jugador.y += jugador_vel
        if keys[pygame.K_SPACE]:
            jugador.disparo()


        for enemigo in enemigos[:]:
            enemigo.movimiento_enemigo(enemigo_vel)
            enemigo.mover_lasers(laser_vel, jugador)
            jugador.mover_lasers(-laser_vel,enemigos)

            if random.randrange(0, 2*60) == 1:
                enemigo.disparo()

            if chocar(enemigo, jugador):
                jugador.vida -= 10 
                puntos -=1
                enemigos.remove(enemigo)
            elif enemigo.y + enemigo.get_altura() > alto:
                niveles -= 1
                enemigos.remove(enemigo)


def menu():
    pygame.mixer.music.load("menu.ogg")
    pygame.mixer.music.play(-1)
    pygame.mixer.music.set_volume(0.03)
    titulo_font = pygame.font.SysFont("comicsans", 55)
    correr = True
    while correr:
        pantalla.blit(MENU,(0,0))
        keys = pygame.key.get_pressed()
        if keys[pygame.K_t]:
            pantalla.blit(TUTORIAL,(0,0))
        pygame.display.update()
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.mixer.stop()
                sys.exit() 
        if keys[pygame.K_RETURN]:
            SpaceInvader2() 
            if event.type == pygame.K_RETURN:
                break
                SpaceInvader2()
    pygame.quit()

menu()




